@@ -6,10 +6,7 @@ qbit::Array - Functions to manipulate dates.
 =cut
 
 package qbit::Date;
-{
-  $qbit::Date::VERSION = '0.9';
-}
-
+$qbit::Date::VERSION = '1.9';
 use strict;
 use warnings;
 use utf8;
@@ -43,7 +40,7 @@ our %TR_HS = (
     db => {
         '>' => sub {sprintf('%04d-%02d-%02d', @{$_[0]})},
         '<' => sub {
-            shift =~ /(\d{4})-(\d{2})-(\d{2})(?:\s+|T\d{1,2}:\d{1,2}:\d{1,2})?/
+            shift =~ /\A(\d{4})-(\d{2})-(\d{2})\z/
               ? [$1, $2, $3, 0, 0, 0]
               : [0, 0, 0, 0, 0, 0];
         },
@@ -53,21 +50,16 @@ our %TR_HS = (
             sprintf("%04d-%02d-%02d %02d:%02d:%02d", @{$_[0]});
         },
         '<' => sub {
-            shift =~ /(\d{4})-(\d{2})-(\d{2})(?:(?:\s+|T)(\d{1,2}):(\d{1,2}):(\d{1,2}))?/
+            shift =~ /\A(\d{4})-(\d{2})-(\d{2})(?:(?:\s+|T)(\d{1,2}):(\d{1,2}):(\d{1,2}))?\z/
               ? [$1, $2, $3, $4, $5, $6]
               : [0, 0, 0, 0, 0, 0];
         },
     },
     sec => {
         '>' => sub {
-            $_[0]->[1] < 1 ? 0 : timelocal(
-                $_[0]->[5] || 0,
-                $_[0]->[4] || 0,
-                $_[0]->[3] || 0,
-                $_[0]->[2],
-                ($_[0]->[1] - 1),
-                ($_[0]->[0] - 1900)
-            );
+            $_[0]->[1] < 1
+              ? 0
+              : timelocal($_[0]->[5] || 0, $_[0]->[4] || 0, $_[0]->[3] || 0, $_[0]->[2], ($_[0]->[1] - 1), $_[0]->[0]);
         },
         '<' => sub {
             my @dt = localtime(shift);
