@@ -1,3 +1,4 @@
+
 =head1 Name
 
 qbit::Exceptions - qbit exceptions
@@ -5,10 +6,7 @@ qbit::Exceptions - qbit exceptions
 =cut
 
 package qbit::Exceptions;
-{
-  $qbit::Exceptions::VERSION = '0.9';
-}
-
+$qbit::Exceptions::VERSION = '1.9';
 =head1 Synopsis
 
 Usage:
@@ -117,15 +115,27 @@ sub try(&;$) {
     my $cur_catch = $catch;
     my $find_catch = !defined($catch) || $catch->[0] eq '::FINALLY::';
 
+    my $first_exception = '';
     if ($@) {
         $@ = Exception::SysDie->new($@)
           unless ref($@) && $@->isa('Exception');
+
+        $first_exception = $@;
+
         while (defined($cur_catch)) {
             last if $cur_catch->[0] eq '::FINALLY::';
             if ($find_catch || $@->isa($cur_catch->[0])) {
                 $find_catch = 1;
                 if (ref($cur_catch->[1]) eq 'CODE') {
-                    $cur_catch->[1]($@);
+                    eval {$cur_catch->[1]($first_exception)};
+
+                    if ($@) {
+                        $find_catch = 0;
+
+                        $@ = Exception::SysDie->new($@)
+                          unless ref($@) && $@->isa('Exception');
+                    }
+
                     last;
                 } else {
                     $cur_catch = $cur_catch->[1];
@@ -141,7 +151,8 @@ sub try(&;$) {
 
     die("Expected semicolon after catch block (" . join(", ", (caller())[1, 2]) . ")\n")
       if defined($cur_catch) && ref($cur_catch) ne 'ARRAY';
-    $cur_catch->[1]() if defined($cur_catch);
+
+    $cur_catch->[1]($first_exception) if defined($cur_catch);
 
     die $@ if $@ && !$find_catch;
 }
@@ -166,10 +177,7 @@ sub throw($) {
 }
 
 package Exception;
-{
-  $Exception::VERSION = '0.9';
-}
-
+$Exception::VERSION = '1.9';
 use strict;
 use warnings;
 use overload '""' => sub {shift->as_string()};
@@ -188,9 +196,7 @@ sub new {
     while (1) {
 
         package DB;
-{
-  $DB::VERSION = '0.9';
-}
+$DB::VERSION = '1.9';
         my ($package, $filename, $line, $subroutine, $hasargs, $wantarray, $evaltext, $is_require, $hints, $bitmask) =
           caller($i);
 
@@ -220,7 +226,6 @@ sub new {
             ? (text => $text->{'text'}, parent => $text)
             : (text => $text)
         ),
-        text      => $text,
         filename  => $caller->{'filename'},
         package   => $caller->{'package'},
         line      => $caller->{'line'},
@@ -260,10 +265,7 @@ sub as_string {
 }
 
 package Exception::SysDie;
-{
-  $Exception::SysDie::VERSION = '0.9';
-}
-
+$Exception::SysDie::VERSION = '1.9';
 use base qw(Exception);
 
 use strict;
@@ -278,15 +280,11 @@ sub new {
 }
 
 package Exception::BadArguments;
-{
-  $Exception::BadArguments::VERSION = '0.9';
-}
+$Exception::BadArguments::VERSION = '1.9';
 use base qw(Exception);
 
 package Exception::Denied;
-{
-  $Exception::Denied::VERSION = '0.9';
-}
+$Exception::Denied::VERSION = '1.9';
 use base qw(Exception);
 
 1;
