@@ -1,7 +1,5 @@
 package Exception::BadArguments::InvalidJSON;
-{
-  $Exception::BadArguments::InvalidJSON::VERSION = '0.9';
-}
+$Exception::BadArguments::InvalidJSON::VERSION = '1.9';
 use base qw(Exception::BadArguments);
 
 =head1 Name
@@ -11,10 +9,7 @@ qbit::StringUtils - Functions to manipulate strings.
 =cut
 
 package qbit::StringUtils;
-{
-  $qbit::StringUtils::VERSION = '0.9';
-}
-
+$qbit::StringUtils::VERSION = '1.9';
 use strict;
 use warnings;
 use utf8;
@@ -380,7 +375,11 @@ sub format_number($%) {
     $number = sprintf("%.${fmt_precision}f", $number)
       if $number =~ /^(-?[\d.]+)e([+-]\d+)$/;    # Convert exponent notation
 
-    setlocale(LC_NUMERIC, "$ENV{'LC_ALL'}.utf8") if defined($ENV{'LC_ALL'});
+    my $old_locale;
+    if (defined($ENV{'LC_ALL'})) {
+        $old_locale = setlocale(LC_NUMERIC);
+        setlocale(LC_NUMERIC, "$ENV{'LC_ALL'}.utf8");
+    }
 
     my $localeconv = localeconv();
     my $half       = 0.50000000000008;
@@ -392,9 +391,11 @@ sub format_number($%) {
         }
     }
 
-    my ($int, $frac_zero, $frac) =
-      $number =~ /^(\d+)(?:[^\d](0*)(\d*))?$/
-      ? (int($1), $2, int($3 || 0))
+    setlocale(LC_NUMERIC, $old_locale) if $old_locale;
+
+    my ($minus, $int, $frac_zero, $frac) =
+      $number =~ /^(-?)(\d+)(?:[^\d](0*)(\d*))?$/
+      ? ($1, int($2), $3, int($4 || 0))
       : throw Exception::BadArguments gettext('Invalid number "%s"', $number);
 
     $frac_zero = '' unless defined($frac_zero);
@@ -418,7 +419,7 @@ sub format_number($%) {
         $int = reverse($int);
     }
 
-    return "$int$frac";
+    return "$minus$int$frac";
 }
 
 =head2 fix_utf
